@@ -103,7 +103,7 @@ class ReconPlot(Plot):
             pass
         else:
             raise RuntimeError("Error: recon_data must be dataframe")
-
+        
         #Retrieve storm data
         if radlim == None:
             lats = recon_data['lat']
@@ -133,7 +133,7 @@ class ReconPlot(Plot):
 
         #Get colormap and level extrema
         cmap,clevs = get_cmap_levels(varname,prop['cmap'],prop['levels'])
-        if varname in ['vmax','sfmr','fl_to_sfc'] and prop['cmap'] in ['category','category_recon']:
+        if varname in ['vmax','sfmr','wspd','fl_to_sfc'] and prop['cmap'] in ['category','category_recon']:
             vmin = min(clevs); vmax = max(clevs)
         else:
             vmin = min(prop['levels']); vmax = max(prop['levels'])
@@ -163,14 +163,13 @@ class ReconPlot(Plot):
         if len(vdms) > 0:
             for vdm_idx in range(len(vdms)):
                 vdm = vdms[vdm_idx]
-                
-                #Plot dot
-                #self.ax.plot(vdm['lon'],vdm['lat'],'o',ms=prop['vdm_ms'],mfc='orange',mec='k',mew=prop['vdm_ms']*0.1,
-                #             transform=ccrs.PlateCarree())
-                
+
                 #Transform coordinates for label
-                a = self.ax.text(vdm['lon'],vdm['lat'],str(int(np.round(vdm['Minimum Sea Level Pressure (hPa)']))),zorder=30,clip_on=True,fontsize=12,fontweight='bold',ha='center',va='center')
-                a.set_path_effects([patheffects.Stroke(linewidth=0.5,foreground='w'),patheffects.Normal()])
+                try:
+                    a = self.ax.text(vdm['lon'],vdm['lat'],str(int(np.round(vdm['Minimum Sea Level Pressure (hPa)']))),zorder=30,clip_on=True,fontsize=12,fontweight='bold',ha='center',va='center')
+                    a.set_path_effects([patheffects.Stroke(linewidth=0.5,foreground='w'),patheffects.Normal()])
+                except:
+                    pass
         
         #--------------------------------------------------------------------------------------
         
@@ -254,7 +253,7 @@ class ReconPlot(Plot):
         cax.yaxis.set_ticks_position('left')
     
         rect_offset = 0.0
-        if prop['cmap'] in ['category','category_recon'] and varname=='sfmr':
+        if prop['cmap'] in ['category','category_recon'] and varname in ['sfmr','wspd']:
             cax.yaxis.set_ticks(np.linspace(min(clevs),max(clevs),len(clevs)))
             cax.yaxis.set_ticklabels(clevs)
             cax2 = cax.twinx()
@@ -436,7 +435,7 @@ class ReconPlot(Plot):
         cax.yaxis.set_ticks_position('left')
     
         rect_offset = 0.0
-        if prop['cmap']=='category' and varname=='sfmr':
+        if prop['cmap'] == 'category' and varname in ['sfmr','wspd']:
             cax.yaxis.set_ticks(np.linspace(0,1,len(clevs)))
             cax.yaxis.set_ticklabels(clevs)
             cax2 = cax.twinx()
