@@ -515,10 +515,14 @@ class Mission():
         summary = ["<tropycal.recon.Mission>"]
         
         #Find maximum wind and minimum pressure
-        max_wspd = np.nanmax(self.hdobs['wspd'])
-        max_pkwnd = np.nanmax(self.hdobs['pkwnd'])
-        max_sfmr = np.nanmax(self.hdobs['sfmr'])
-        min_psfc = np.nanmin(self.hdobs['p_sfc'])
+        array = [val for i,val in enumerate(self.hdobs['sfmr']) if 'sfmr' not in self.hdobs['flag'].values[i]]
+        max_sfmr = np.nanmax(array) if all_nan(array) == False else np.nan
+        array = [val for i,val in enumerate(self.hdobs['p_sfc']) if 'p_sfc' not in self.hdobs['flag'].values[i]]
+        min_psfc = np.nanmin(array) if all_nan(array) == False else np.nan
+        array = [val for i,val in enumerate(self.hdobs['wspd']) if 'wspd' not in self.hdobs['flag'].values[i]]
+        max_wspd = np.nanmax(array) if all_nan(array) == False else np.nan
+        array = [val for i,val in enumerate(self.hdobs['pkwnd']) if 'pkwnd' not in self.hdobs['flag'].values[i]]
+        max_pkwnd = np.nanmax(array) if all_nan(array) == False else np.nan
         time_range = [pd.to_datetime(t) for t in (np.nanmin(self.hdobs['time']),np.nanmax(self.hdobs['time']))]
 
         #Add summary text
@@ -561,6 +565,15 @@ class Mission():
         self.start_time = pd.to_datetime(np.nanmin(self.hdobs['time']))
         self.end_time = pd.to_datetime(np.nanmax(self.hdobs['time']))
         
+        #Get sources
+        if 'source' in data.keys():
+            self.source = data['source']
+        else:
+            if self.start_time.year <= 2005 and self.start_time.year >= 1989:
+                self.source = ['National Hurricane Center (NHC)',"UCAR's Tropical Cyclone Guidance Project (TCGP)"]
+            else:
+                self.source = 'National Hurricane Center (NHC)'
+        
         #Retrieve attributes
         self.attrs = {
             'aircraft':data['aircraft'],
@@ -568,6 +581,7 @@ class Mission():
             'mission_id':mission_id,
             'start_time': pd.to_datetime(np.nanmin(self.hdobs['time'])),
             'end_time': pd.to_datetime(np.nanmax(self.hdobs['time'])),
+            'source': self.source,
         }
         
         #Add status for mission
